import {Control, ControlView} from "./control"
import {StyleSheetLike} from "core/dom"
import * as p from "core/properties"
import inputs_css from "styles/widgets/inputs.css"
import checkbox_css from "styles/widgets/checkbox.css"

export abstract class ToggleInputGroupView extends ControlView {
  override model: ToggleInputGroup

  protected _inputs: HTMLInputElement[]
  *controls() {
    yield* this._inputs
  }

  override connect_signals(): void {
    super.connect_signals()

    const {labels, inline} = this.model.properties
    this.on_change([labels, inline], () => this.render())
  }

  override styles(): StyleSheetLike[] {
    return [...super.styles(), inputs_css, checkbox_css]
  }
}

export namespace ToggleInputGroup {
  export type Attrs = p.AttrsOf<Props>

  export type Props = Control.Props & {
    labels: p.Property<string[]>
    inline: p.Property<boolean>
  }
}

export interface ToggleInputGroup extends ToggleInputGroup.Attrs {}

export abstract class ToggleInputGroup extends Control {
  override properties: ToggleInputGroup.Props & {active: p.Property<unknown>}
  override __view_type__: ToggleInputGroupView

  constructor(attrs?: Partial<ToggleInputGroup.Attrs>) {
    super(attrs)
  }

  static {
    this.define<ToggleInputGroup.Props>(({Boolean, String, Array}) => ({
      labels: [ Array(String), [] ],
      inline: [ Boolean, false ],
    }))
  }
}
